/* Hash function characterization.
 *
 * Copyright 2022-2025 Joaquin M Lopez Munoz.
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 *
 * See https://www.boost.org/libs/unordered for library home page.
 */

#ifndef BOOST_UNORDERED_HASH_TRAITS_HPP
#define BOOST_UNORDERED_HASH_TRAITS_HPP

#include <boost/config/header_deprecated.hpp>
#include <boost/container_hash/hash_is_avalanching.hpp>

BOOST_HEADER_DEPRECATED("<boost/container_hash/hash_is_avalanching.hpp>")

namespace boost{
namespace unordered{

using boost::hash_is_avalanching;

} /* namespace unordered */
} /* namespace boost */

#endif
