<banner>Welcome to the Phusion Passenger Nginx module installer, v<%= @version %>.</banner>

This installer will guide you through the entire installation process. It
shouldn't take more than 5 minutes in total.

Here's what you can expect from the installation process:

 <b>1.</b> This installer will compile and install Nginx with Passenger support.
 <b>2.</b> You'll learn how to configure Passenger in Nginx.
 <b>3.</b> You'll learn how to deploy a Ruby on Rails application.

Don't worry if anything goes wrong. This installer will advise you on how to
solve any problems.

<b>Press Enter to continue, or Ctrl-C to abort.</b>
