<red>An unexpected problem occurred</red>

This program must be able to create this directory:

  <b><%= @dir %></b>

However, some completely unexpected filesystem error occurred. Unfortunately,
this program can't figure out what's wrong, so it's up to you now. You can
find the system error message below. Please find out what's wrong, fix the
problems, and re-run this program. If you don't know it either, please contact
your operating system vendor for support, or consult your operating system's
manual.

The error message is as follows:

  <b><%= @exception %></b>