<banner>Your system does not have a lot of virtual memory</banner>

Compiling <%= PROGRAM_NAME %> works best when you have at least <%= @required %> MB of virtual
memory. However your system only has <%= @current %> MB of total virtual memory (<%= @ram %> MB
RAM, <%= @swap %> MB swap). It is recommended that you temporarily add more swap space
before proceeding. You can do it as follows:

  <b>sudo dd if=/dev/zero of=/swap bs=1M count=1024
  sudo mkswap /swap
  sudo swapon /swap</b>

See also <b>https://wiki.archlinux.org/index.php/Swap</b> for more information about
the swap file on Linux.

If you cannot activate a swap file (e.g. because you're on OpenVZ, or if you
don't have root privileges) then you should install <%= PROGRAM_NAME %> through
DEB/RPM packages. For more information, please refer to our installation
documentation:

  <%= @install_doc_url %>

<b>Press Ctrl-C to abort this installer (recommended).</b>
<b>Press Enter if you want to continue with installation anyway.</b>
