<red>WARNING:</red> <yellow>Apache doesn't seem to be compiled with the 'prefork', 'worker' or 'event' MPM</yellow>

Phusion Passenger has only been tested on Apache with the 'prefork', the
'worker' and the 'event' MPM. Your Apache installation is compiled with
the '<%= @current_mpm %>' MPM. We recommend you to abort this installer and to recompile
Apache with either the 'prefork', the 'worker' or the 'event' MPM.

<b>Press Ctrl-C to abort this installer (recommended).</b>
<b>Press Enter if you want to continue with installation anyway.</b>
