log_format debug '[$time_local] $msec  "$request" $status conn=$connection sent=$bytes_sent body_sent=$body_bytes_sent';
include '<%= PhusionPassenger.resources_dir %>/mime.types';

passenger_root '<%= PhusionPassenger.install_spec %>';
passenger_abort_on_startup_error on;
passenger_ctl pidfiles_to_delete_on_exit '["<%= "#{@working_dir}/temp_dir_toucher.pid" %>"]';
passenger_ctl integration_mode standalone;
passenger_ctl standalone_engine nginx;
passenger_user_switching off;

<% if Process.euid == 0 %>
    <% if @options[:ruby] %>
        passenger_ruby <%= @options[:ruby] %>;
    <% else %>
        passenger_ruby <%= PlatformInfo.ruby_command %>;
    <% end %>
    <% if @options[:user] %>
        passenger_user <%= @options[:user] %>;
        passenger_default_user <%= @options[:user] %>;
    <% else %>
        passenger_user <%= current_user %>;
        passenger_default_user <%= current_user %>;
    <% end %>
<% end %>

<%= nginx_http_option(:socket_backlog) %>
<%= nginx_http_option(:python) %>
<%= nginx_http_option(:nodejs) %>
<%= nginx_http_option(:log_level) %>
<%= nginx_http_option(:disable_log_prefix) %>
<%= nginx_http_option(:max_pool_size) %>
<%= nginx_http_option(:pool_idle_time) %>
<%= nginx_http_option(:max_preloader_idle_time) %>
<%= nginx_http_option(:turbocaching) %>
<%= nginx_http_option(:old_routing) %>
<%= nginx_http_option(:instance_registry_dir) %>
<%= nginx_http_option(:spawn_dir) %>
<%= nginx_http_option(:disable_security_update_check) %>
<%= nginx_http_option(:security_update_check_proxy) %>
<%= nginx_http_option(:disable_anonymous_telemetry) %>
<%= nginx_http_option(:anonymous_telemetry_proxy) %>
<%= nginx_http_option(:data_buffer_dir) %>
<%= nginx_http_option(:core_file_descriptor_ulimit) %>
<%= nginx_http_option(:admin_panel_url) %>
<%= nginx_http_option(:admin_panel_auth_type) %>
<%= nginx_http_option(:admin_panel_username) %>
<%= nginx_http_option(:admin_panel_password) %>
<% @options[:ctls].each do |ctl| %>
    passenger_ctl '<%= ctl.split("=", 2)[0] %>' '<%= ctl.split("=", 2)[1] %>';
<% end %>
