<% @other_installs.each do |result| %>
 * To compile against <yellow>Apache <%= result.version %> (<%= result.apxs2 %>)</yellow>:
   Re-run this installer with: <b>--apxs2-path "<%= result.apxs2 %>"</b>
<% end %>
You may also want to read the installation guide and the troubleshooting guide:

  https://www.phusionpassenger.com/library/install/apache/
  https://www.phusionpassenger.com/library/admin/apache/troubleshooting/

If you keep having problems installing, please visit the following website for
support:

  <%= SUPPORT_URL %>