body {
  margin-top: 2em;
  margin-bottom: 2em;
}

@media (max-width: 991px) {
  body {
    margin-top: 1em;
    margin-bottom: 1em;
  }
}

.h1, .h2, .h3,
h1, h2, h3 {
  font-family: Lato, Helvetica, Arial, sans-serif;
  margin-top: 24px;
  margin-bottom: 16px;
  line-height: 1.25;
  font-weight: 700;
}

.h2, h2 {
  font-size: 190%;
}

.h3, h3 {
  font-size: 120%;
  color: #444;
}

.page-title {
  font-size: 250%;
  font-weight: 300;
  color: #999;
  margin: 0 0 1em 0;
}

.page-system-components-container {
  position: relative;
  background: #f6f6f6;
  margin: 0 0 2em 0;
  padding: 2em;
}

.page-system-components-container .collapse-button > a {
  position: absolute;
  right: 0;
  top: 0;
  background: #dcdcdc;
  font-size: 90%;
  text-decoration: none;
  padding: 0.5em;
}

.page-main-tabs > .nav-tabs > li > a {
  font-weight: bold;
}

.page-main-tabs > .tab-content {
  margin-top: 2em;
}

footer {
    font-size: 14px;
    color: #7F7F7F;
    border-top: 1px solid #E3E3E3;
    margin-top: 50px;
    padding: 40px 0px;
}

footer div {
    max-width: 1000px;
    margin-left: auto;
    margin-right: auto;
    text-align: center;
}

pre {
  color: #526379;
  border: none;
  font-size: 12px;
}

code {
  background: #e5eef9;
  color: #333;
}


.list-with-extra-spacing > li {
  margin-top: 0.5em;
}


.problem-causes th {
  background: #f5f5f5;
}

.problem-causes .solution-preview {
  white-space: nowrap;
}

.problem-causes .solution-description td {
  padding: 1em;
  background: #d9edf7;
}

.problem-causes .solution-description pre {
  background: white;
}

.problem-causes .solution-description .multiple-solutions > h3:first-child {
  margin-top: 0;
}
