<red>Permission problems</red>

This installer must be able to write to the following directory:

  <b><%= @dir %></b>

But it can't do that, despite the fact that it's running as root. There's
probably a permission problem, or (if you're running on a
RedHat/CentOS-based Linux distribution) SELinux might be interfering.
Please fix the permissions for the aforementioned directory and/or disable
SELinux, and re-run this installer.