<banner>Compile the agent with optimizations?</banner>

Compiling the agent with optimizations will make <%= PROGRAM_NAME %> faster, but
it will take longer to compile and it requires <b>at least 2 GB of memory</b>.
<% if @total_ram %>(You have <%= @total_ram %> GB memory.)<% end -%>