<red>The necessary source files for compiling Nginx are not installed.</red>
<% if PhusionPassenger.packaging_method == "deb" %>
Please install them with:

  <b>sudo apt-get install <%= DEB_DEV_PACKAGE %></b>
<% elsif PhusionPassenger.packaging_method == "rpm" %>
Please install them with:

  <b>sudo yum install <%= RPM_DEV_PACKAGE %>-<%= VERSION_STRING %></b>
<% else %>
Please ask your operating system vendor how to solve this problem.
<% end %>