<red><b>PCRE could not be downloaded</b></red>

Nginx requires PCRE for its rewrite module, so this installer will attempt to
install Nginx without the rewrite module.

If you want to make use of Nginx's rewrite module, please install PCRE manually
by downloading it from:

  http://www.pcre.org/

Press ENTER to continue, or Ctrl-C to abort.