#  Phusion Passenger - https://www.phusionpassenger.com/
#  Copyright (c) 2010-2025 Asynchronous B.V.
#
#  "Passenger", "Phusion Passenger" and "Union Station" are registered
#  trademarks of Asynchronous B.V.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#  THE SOFTWARE.

class Object # :nodoc:
  @@benchmark_results = {}

  def b!(name)
    time1 = Time.now
    begin
      yield
    ensure
      time2 = Time.now
      @@benchmark_results[name] = 0 unless @@benchmark_results.has_key?(name)
      @@benchmark_results[name] += time2 - time1
    end
  end

  def benchmark_report(main = nil)
    total = 0
    if main.nil?
      @@benchmark_results.each_value do |time|
        total += time
      end
    else
      total = @@benchmark_results[main]
    end
    @@benchmark_results.each_pair do |name, time|
      printf "%-12s: %.4f (%.2f%%)\n", name, time, time / total * 100
    end
    printf "-- Total: %.4f\n", total
  end
end
