<banner>Nginx with Passenger support was successfully installed.</banner>

Please edit your Nginx configuration file<% if @config_file %> (probably <b><%= @config_file %></b>)<% end %>,
and set the <b>passenger_root</b> and <b>passenger_ruby</b> configuration options in the
<b>'http'</b> block, like this:

  <b>http {
      ...
      passenger_root <%= @passenger_root %>;
      passenger_ruby <%= @ruby %>;
      ...
  }</b>

After you (re)start Nginx, you are ready to deploy any number of web
applications on Nginx.

<b>Press ENTER to continue.</b>
