<banner>Deploying a web application</banner>

To learn how to deploy a web app on Passenger, please follow the deployment
guide:

  <yellow><%= @deployment_guide_url %></yellow>

Enjoy Phusion Passenger, a product of Phusion® (<b><%= @phusion_website %></b>) :-)
<b><%= @passenger_website %></b>

Passenger® is a registered trademark of Asynchronous B.V.
