<banner>Almost there!</banner>

Please edit your Apache configuration file, and add these lines:

   <b><%= @snippet.split("\n").join("\n   ") %></b>

After you restart Apache, you are ready to deploy any number of web
applications on Apache, with a minimum amount of configuration!

<b>Press ENTER when you are done editing.</b>
