(<yellow>Tip:</yellow> you can also re-run this installer with <yellow>--no-update-config</yellow>, to prevent
it from automatically updating your Apache configuration files. This way, the
installer will not need as many privileges. The installer will then tell you
how you must update the config files. You will then have to edit the config
files manually.)