<red><b>Unable to download or extract Nginx source tarball</b></red>

Possible reasons:

 <b>* You are not connected to the Internet.</b>
   If this is the case, then please connect to the Internet and try again.

 <b>* The URL that this installer tried to download from no longer exists.</b>
   If this is the case, then please download and extract the Nginx source
   manually. Then restart this installer, and choose option 2 ("No: I want
   to customize my Nginx installation").

 <b>* The download was corrupted.</b>
   Please re-run this installer; it will re-download Nginx. If this still
   fails, then please download and extract the Nginx source manually. Then
   restart this installer, and choose option 2 ("No: I want to customize my
   Nginx installation").

 <b>* This installer is not able to write to /tmp.</b>
   Please fix the permissions for this directory, or run this installer as <b>root</b>.