body {
    font-family: Arial, Sans-Serif;
    font-size: 13px;
    color: #222222;
    margin: 0;
}
.column {
    max-width: 1000px;
    margin-left: auto;
    margin-right: auto;
    text-align: center;
}
header {
    margin-top: 50px;
    border-bottom: 1px solid #E3E3E3;
    margin-bottom: 45px;
}
footer {
    font-size: 14px;
    color: #7F7F7F;
    border-top: 1px solid #E3E3E3;
    margin-top: 50px;
    padding: 40px 0px;
}
h1 {
    font-size: 30px;
    margin-top: 30px;
    margin-bottom: 10px;
}
.subtitle {
    margin-top: 0px;
    margin-bottom: 110px;
    font-size: 20px;
}
#operator_info {
    display: none;
}
#show_operator_info {
    font-size: 17px;
    font-weight: normal;
}
.left {
    text-align: left;
    padding: 8px;
}
h3 {
    margin-top: 30px;
    margin-bottom: 10px;
    font-size: 23px;
}
ul {
    padding-left: 16px;
}
a, li {
    font-weight: bold;
    color: #1781BF;
    text-decoration: none;
}
.error {
    font-weight: bold;
    background: #E6F3FC;
    border-radius: 5px;
    padding: 7px 12px;
}
.error.block {
    display: block;
}
.bold {
    font-weight: bold !important;
}
pre {
    margin: 0px;
    overflow-x: auto;
    white-space: pre-wrap;
    word-break: break-all; 
}
dt {
    margin-top: 16px;
    font-weight: bold;
}
dd {
    margin-left: 0px;
}
.plain {
    color: inherit;
    font-weight: inherit;
}
 
#content {
    overflow-y: scroll;
    height: 800px;
}
