# Autogenerated by dev/index_cxx_dependencies.rb
CXX_DEPENDENCY_MAP =
{"src/agent/AgentMain.cpp"=>
  ["src/agent/MainFunctions.h",
   "src/cxx_supportlib/Constants.h"],
 "src/agent/Core/ApiServer.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/AbstractSession.h"=>
  ["src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/BasicGroupInfo.h"=>
  ["src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/BasicProcessInfo.h"=>
  ["src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Common.h"=>
  ["src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Context.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/InitializationAndShutdown.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/InternalUtils.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/LifetimeAndBasics.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/Miscellaneous.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/OutOfBandWork.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/ProcessListManagement.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/SessionManagement.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/SpawningAndRestarting.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/StateInspection.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Group/Verification.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Implementation.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Group/InitializationAndShutdown.cpp",
   "src/agent/Core/ApplicationPool/Group/InternalUtils.cpp",
   "src/agent/Core/ApplicationPool/Group/LifetimeAndBasics.cpp",
   "src/agent/Core/ApplicationPool/Group/Miscellaneous.cpp",
   "src/agent/Core/ApplicationPool/Group/OutOfBandWork.cpp",
   "src/agent/Core/ApplicationPool/Group/ProcessListManagement.cpp",
   "src/agent/Core/ApplicationPool/Group/SessionManagement.cpp",
   "src/agent/Core/ApplicationPool/Group/SpawningAndRestarting.cpp",
   "src/agent/Core/ApplicationPool/Group/StateInspection.cpp",
   "src/agent/Core/ApplicationPool/Group/Verification.cpp",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Pool/AnalyticsCollection.cpp",
   "src/agent/Core/ApplicationPool/Pool/GarbageCollection.cpp",
   "src/agent/Core/ApplicationPool/Pool/GeneralUtils.cpp",
   "src/agent/Core/ApplicationPool/Pool/GroupUtils.cpp",
   "src/agent/Core/ApplicationPool/Pool/InitializationAndShutdown.cpp",
   "src/agent/Core/ApplicationPool/Pool/Miscellaneous.cpp",
   "src/agent/Core/ApplicationPool/Pool/ProcessUtils.cpp",
   "src/agent/Core/ApplicationPool/Pool/StateInspection.cpp",
   "src/agent/Core/ApplicationPool/Process.cpp",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/ErrorRenderer.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/StrIntTools/Template.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Options.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/AnalyticsCollection.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/GarbageCollection.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/GeneralUtils.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/GroupUtils.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/InitializationAndShutdown.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/Miscellaneous.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/ProcessUtils.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Pool/StateInspection.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Process.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Process.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Session.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/Socket.h"=>
  ["src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ApplicationPool/TestSession.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Config.h"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ConfigChange.cpp"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Config.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ConfigChange.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/AppResponse.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/BufferBody.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/CheckoutSession.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/ErrorRenderer.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/StrIntTools/Template.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Client.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Config.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Config.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/ForwardResponse.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Hooks.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Implementation.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/BufferBody.cpp",
   "src/agent/Core/Controller/CheckoutSession.cpp",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.cpp",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/ForwardResponse.cpp",
   "src/agent/Core/Controller/Hooks.cpp",
   "src/agent/Core/Controller/InitRequest.cpp",
   "src/agent/Core/Controller/InitializationAndShutdown.cpp",
   "src/agent/Core/Controller/InternalUtils.cpp",
   "src/agent/Core/Controller/Miscellaneous.cpp",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/SendRequest.cpp",
   "src/agent/Core/Controller/StateInspection.cpp",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/ErrorRenderer.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/StrIntTools/Template.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/InitRequest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/InitializationAndShutdown.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/InternalUtils.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Miscellaneous.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/Request.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/SendRequest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/StateInspection.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/Controller/TurboCaching.h"=>
  ["src/agent/Core/ResponseCache.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/CoreMain.cpp"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Config.h",
   "src/agent/Core/ConfigChange.cpp",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/OptionParser.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/MainFunctions.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Initialization.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/BackgroundEventLoop.cpp",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/SubComponentUtils.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/FileTools/PathSecurityCheck.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/Autocast.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/AcceptLoadBalancer.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/OptionParsing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/OptionParser.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/JsonTools/Autocast.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/OptionParsing.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/ResponseCache.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/Core/SecurityUpdateChecker.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Config.h"=>
  ["src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h"=>
  [],
 "src/agent/Core/SpawningKit/Context.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/DirectSpawner.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/DummySpawner.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/ErrorRenderer.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/StrIntTools/Template.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Exceptions.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Factory.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Handshake/Perform.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Handshake/Prepare.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Handshake/Session.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Handshake/WorkDir.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Journey.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/PipeWatcher.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Result.h"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h"=>
  [],
 "src/agent/Core/SpawningKit/SmartSpawner.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/Spawner.h"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/SpawningKit/UserSwitchingRules.h"=>
  ["src/agent/Core/SpawningKit/Context.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Core/TelemetryCollector.h"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/ExecHelper/ExecHelperMain.cpp"=>
  ["src/agent/MainFunctions.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/OptionParsing.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/FileReadHelper/FileReadHelperMain.cpp"=>
  ["src/agent/MainFunctions.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/OptionParsing.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/MainFunctions.h"=>
  [],
 "src/agent/Shared/ApiAccountUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/ApiServerUtils.h"=>
  ["src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/ApplicationPoolApiKey.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/Fundamentals/AbortHandler.cpp"=>
  ["src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/Fundamentals/AbortHandler.h"=>
  [],
 "src/agent/Shared/Fundamentals/Implementation.cpp"=>
  ["src/agent/Shared/Fundamentals/AbortHandler.cpp",
   "src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Initialization.cpp",
   "src/agent/Shared/Fundamentals/Initialization.h",
   "src/agent/Shared/Fundamentals/Utils.cpp",
   "src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/Fundamentals/Initialization.cpp"=>
  ["src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Initialization.h",
   "src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/Fundamentals/Initialization.h"=>
  ["src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Shared/Fundamentals/Utils.cpp"=>
  ["src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/Shared/Fundamentals/Utils.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/SpawnEnvSetupper/SpawnEnvSetupperMain.cpp"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/MainFunctions.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/SystemMetrics/SystemMetricsMain.cpp"=>
  ["src/agent/MainFunctions.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/TempDirToucher/TempDirToucherMain.cpp"=>
  ["src/agent/MainFunctions.h",
   "src/cxx_supportlib/Constants.h"],
 "src/agent/Watchdog/AgentWatcher.cpp"=>
  [],
 "src/agent/Watchdog/ApiServer.h"=>
  ["src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Watchdog/Config.h"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Config.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/agent/Watchdog/ApiServer.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Watchdog/CoreWatcher.cpp"=>
  ["src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/agent/Watchdog/InstanceDirToucher.cpp"=>
  ["src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/agent/Watchdog/WatchdogMain.cpp"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Config.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/OptionParser.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/MainFunctions.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Initialization.h",
   "src/agent/Shared/Fundamentals/Utils.h",
   "src/agent/Watchdog/AgentWatcher.cpp",
   "src/agent/Watchdog/ApiServer.h",
   "src/agent/Watchdog/Config.h",
   "src/agent/Watchdog/CoreWatcher.cpp",
   "src/agent/Watchdog/InstanceDirToucher.cpp",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/FileTools/PathSecurityCheck.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/Autocast.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ContainerHelpers.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/OptionParsing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Bucket.cpp"=>
  ["src/apache2_module/Bucket.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Bucket.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/CBindings.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Hooks.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Config.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/AutoGeneratedDefinitions.cpp",
   "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp",
   "src/apache2_module/ConfigGeneral/AutoGeneratedSetterFuncs.cpp",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/ConfigGeneral/ManifestGeneration.h",
   "src/apache2_module/ConfigGeneral/SetterFuncs.h",
   "src/apache2_module/DirConfig/AutoGeneratedCreateFunction.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedMergeFunction.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/JsonTools/Autocast.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Config.h"=>
  ["src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/ConfigGeneral/AutoGeneratedDefinitions.cpp"=>
  [],
 "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/ConfigGeneral/ManifestGeneration.h",
   "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/ConfigGeneral/AutoGeneratedSetterFuncs.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/ConfigGeneral/Common.h"=>
  [],
 "src/apache2_module/ConfigGeneral/ManifestGeneration.h"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/ConfigGeneral/ManifestGeneration.h",
   "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/ConfigGeneral/SetterFuncs.h"=>
  [],
 "src/apache2_module/DirConfig/AutoGeneratedCreateFunction.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/DirConfig/AutoGeneratedHeaderSerialization.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/ConfigGeneral/ManifestGeneration.h",
   "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/DirConfig/AutoGeneratedMergeFunction.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/DirConfig/AutoGeneratedStruct.h"=>
  ["src/apache2_module/ConfigGeneral/Common.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/apache2_module/DirectoryMapper.h"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Hooks.cpp"=>
  ["src/apache2_module/Bucket.h",
   "src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/DirConfig/AutoGeneratedHeaderSerialization.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/DirectoryMapper.h",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/ClassUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ReleaseableScopedPointer.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/WatchdogLauncher.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/Hooks.h"=>
  [],
 "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp"=>
  ["src/apache2_module/Config.h",
   "src/apache2_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.cpp",
   "src/apache2_module/ConfigGeneral/Common.h",
   "src/apache2_module/ConfigGeneral/ManifestGeneration.h",
   "src/apache2_module/DirConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/DirConfig/AutoGeneratedStruct.h",
   "src/apache2_module/ServerConfig/AutoGeneratedManifestGeneration.cpp",
   "src/apache2_module/ServerConfig/AutoGeneratedStruct.h",
   "src/apache2_module/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/apache2_module/ServerConfig/AutoGeneratedStruct.h"=>
  ["src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/apache2_module/Utils.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/apache2_module/mod_passenger.c"=>
  [],
 "src/cxx_supportlib/Algorithms/Hasher.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h"],
 "src/cxx_supportlib/Algorithms/Hasher.h"=>
  [],
 "src/cxx_supportlib/Algorithms/MovingAverage.h"=>
  ["src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/AppLocalConfigFileUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/AppTypeDetector/CBindings.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/AppTypeDetector/CBindings.h",
   "src/cxx_supportlib/AppTypeDetector/Detector.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/AppTypeDetector/CBindings.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/AppTypeDetector/Detector.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/BackgroundEventLoop.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/BackgroundEventLoop.h"=>
  [],
 "src/cxx_supportlib/ConfigKit/AsyncUtils.h"=>
  ["src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/Common.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/ConfigKit.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ConfigKit/DummyTranslator.h"=>
  ["src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/PrefixTranslator.h"=>
  ["src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/Schema.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ConfigKit/SchemaUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ConfigKit/Store.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ConfigKit/SubComponentUtils.h"=>
  ["src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/TableTranslator.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/Translator.h"=>
  ["src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ConfigKit/Utils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/Constants.h"=>
  [],
 "src/cxx_supportlib/DataStructures/HashedStaticString.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/DataStructures/LString.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/DataStructures/LString.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/DataStructures/StringKeyTable.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/DataStructures/StringMap.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/Exceptions.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Exceptions.h"=>
  ["src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileDescriptor.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileTools/FileManip.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileTools/FileManip.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/FileTools/PathManip.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileTools/PathManip.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/FileTools/PathManipCBindings.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileTools/PathManipCBindings.h"=>
  [],
 "src/cxx_supportlib/FileTools/PathSecurityCheck.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/FileTools/PathSecurityCheck.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/FileTools/PathSecurityCheck.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/Hooks.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/IOTools/BufferedIO.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/IOTools/IOUtils.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/IOTools/IOUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/IOTools/MessageIO.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/IOTools/MessageSerialization.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/InstanceDirectory.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Integrations/LibevJsonUtils.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/JsonTools/Autocast.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/JsonTools/CBindings.cpp"=>
  ["src/cxx_supportlib/JsonTools/Autocast.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/JsonTools/CBindings.h"=>
  [],
 "src/cxx_supportlib/JsonTools/JsonUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/LoggingKit/Assert.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/LoggingKit/Config.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/LoggingKit/Context.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/LoggingKit/Forward.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/LoggingKit/Implementation.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/LoggingKit/Logging.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/LoggingKit/LoggingKit.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/LveLoggingDecorator.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/MemoryKit/mbuf.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp"],
 "src/cxx_supportlib/MemoryKit/mbuf.h"=>
  ["src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/MemoryKit/palloc.cpp"=>
  ["src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/MemoryKit/palloc.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ProcessManagement/Ruby.cpp"=>
  ["src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/ProcessManagement/Ruby.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ProcessManagement/Ruby.h"=>
  [],
 "src/cxx_supportlib/ProcessManagement/Spawn.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ProcessManagement/Spawn.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ProcessManagement/Utils.cpp"=>
  ["src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h"],
 "src/cxx_supportlib/ProcessManagement/Utils.h"=>
  [],
 "src/cxx_supportlib/RandomGenerator.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ResourceLocator.h"=>
  ["src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SafeLibev.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp"],
 "src/cxx_supportlib/SecurityKit/Crypto.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SecurityKit/Crypto.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/SecurityKit/MemZeroGuard.h"=>
  [],
 "src/cxx_supportlib/ServerKit/AcceptLoadBalancer.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/Channel.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/Client.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/ClientRef.h"=>
  [],
 "src/cxx_supportlib/ServerKit/Config.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/Context.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/CookieUtils.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ServerKit/Errors.h"=>
  ["src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/FdSinkChannel.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/FdSourceChannel.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/FileBufferedChannel.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/HeaderTable.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ServerKit/Hooks.h"=>
  [],
 "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h"=>
  ["src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h"=>
  [],
 "src/cxx_supportlib/ServerKit/HttpClient.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/HttpHeaderParser.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ServerKit/HttpRequest.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/HttpRequestRef.h"=>
  [],
 "src/cxx_supportlib/ServerKit/HttpServer.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/Implementation.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/ServerKit/Server.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/ServerKit/llerrors.h"=>
  ["src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/llhttp.c"=>
  ["src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/llhttp.h"=>
  [],
 "src/cxx_supportlib/ServerKit/llhttp_api.c"=>
  ["src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/llhttp_http.c"=>
  [],
 "src/cxx_supportlib/ServerKit/llversion.h"=>
  ["src/cxx_supportlib/ServerKit/llhttp.h"],
 "src/cxx_supportlib/ServerKit/url_parser.c"=>
  ["src/cxx_supportlib/ServerKit/url_parser.h"],
 "src/cxx_supportlib/ServerKit/url_parser.h"=>
  [],
 "src/cxx_supportlib/StaticString.h"=>
  ["src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/StrIntTools/DateParsing.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/StrIntTools/StrIntUtils.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/StrIntTools/StrIntUtils.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/StrIntTools/StrIntUtilsNoStrictAliasing.cpp"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/StrIntTools/StringScanning.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/StrIntTools/Template.h"=>
  ["src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/SystemTools/ContainerHelpers.h"=>
  ["src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h"=>
  ["src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SystemTools/SystemTime.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SystemTools/SystemTime.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SystemTools/UserDatabase.cpp"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/SystemTools/UserDatabase.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/Utils.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/AnsiColorConstants.h"=>
  [],
 "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h"=>
  [],
 "src/cxx_supportlib/Utils/BlockingQueue.h"=>
  [],
 "src/cxx_supportlib/Utils/CachedFileStat.cpp"=>
  ["src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/CachedFileStat.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/CachedFileStat.h"=>
  [],
 "src/cxx_supportlib/Utils/CachedFileStat.hpp"=>
  ["src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/ClassUtils.h"=>
  [],
 "src/cxx_supportlib/Utils/Curl.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/FastStringStream.h"=>
  ["src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/Utils/FileChangeChecker.h"=>
  ["src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/HttpConstants.h"=>
  [],
 "src/cxx_supportlib/Utils/IniFile.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/Lock.h"=>
  [],
 "src/cxx_supportlib/Utils/MessagePassing.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/OptionParsing.h"=>
  [],
 "src/cxx_supportlib/Utils/ReleaseableScopedPointer.h"=>
  [],
 "src/cxx_supportlib/Utils/ScopeGuard.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp"],
 "src/cxx_supportlib/Utils/SpeedMeter.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/Timer.h"=>
  ["src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/Utils/VariantMap.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/WatchdogLauncher.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/ClassUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/WatchdogLauncher.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/WatchdogLauncher.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/ClassUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/WrapperRegistry/CBindings.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/WrapperRegistry/CBindings.h"=>
  [],
 "src/cxx_supportlib/WrapperRegistry/Entry.h"=>
  ["src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/WrapperRegistry/Registry.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/oxt/backtrace.hpp"=>
  ["src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp"],
 "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp"=>
  [],
 "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp"=>
  [],
 "src/cxx_supportlib/oxt/detail/context.hpp"=>
  ["src/cxx_supportlib/oxt/spin_lock.hpp"],
 "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp"=>
  [],
 "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp"=>
  [],
 "src/cxx_supportlib/oxt/dynamic_thread_group.hpp"=>
  ["src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp"],
 "src/cxx_supportlib/oxt/implementation.cpp"=>
  ["src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/cxx_supportlib/oxt/initialize.hpp"=>
  [],
 "src/cxx_supportlib/oxt/macros.hpp"=>
  [],
 "src/cxx_supportlib/oxt/spin_lock.hpp"=>
  [],
 "src/cxx_supportlib/oxt/system_calls.cpp"=>
  ["src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp"],
 "src/cxx_supportlib/oxt/system_calls.hpp"=>
  ["src/cxx_supportlib/oxt/macros.hpp"],
 "src/cxx_supportlib/oxt/thread.hpp"=>
  ["src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp"],
 "src/cxx_supportlib/oxt/tracable_exception.hpp"=>
  ["src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp"],
 "src/nginx_module/ConfigGeneral/AutoGeneratedDefinitions.c"=>
  [],
 "src/nginx_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.c"=>
  ["src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.h"],
 "src/nginx_module/ConfigGeneral/AutoGeneratedSetterFuncs.c"=>
  [],
 "src/nginx_module/ConfigGeneral/ManifestGeneration.c"=>
  ["src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.h",
   "src/nginx_module/Configuration.h",
   "src/nginx_module/LocationConfig/AutoGeneratedStruct.h",
   "src/nginx_module/MainConfig/AutoGeneratedStruct.h"],
 "src/nginx_module/ConfigGeneral/ManifestGeneration.h"=>
  ["src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h"],
 "src/nginx_module/Configuration.c"=>
  ["src/cxx_supportlib/AppTypeDetector/CBindings.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "src/nginx_module/ConfigGeneral/AutoGeneratedDefinitions.c",
   "src/nginx_module/ConfigGeneral/AutoGeneratedManifestDefaultsInitialization.c",
   "src/nginx_module/ConfigGeneral/AutoGeneratedSetterFuncs.c",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.c",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.h",
   "src/nginx_module/Configuration.h",
   "src/nginx_module/ContentHandler.h",
   "src/nginx_module/LocationConfig/AutoGeneratedCreateFunction.c",
   "src/nginx_module/LocationConfig/AutoGeneratedHeaderSerialization.c",
   "src/nginx_module/LocationConfig/AutoGeneratedManifestGeneration.c",
   "src/nginx_module/LocationConfig/AutoGeneratedMergeFunction.c",
   "src/nginx_module/LocationConfig/AutoGeneratedStruct.h",
   "src/nginx_module/MainConfig/AutoGeneratedCreateFunction.c",
   "src/nginx_module/MainConfig/AutoGeneratedManifestGeneration.c",
   "src/nginx_module/MainConfig/AutoGeneratedStruct.h"],
 "src/nginx_module/Configuration.h"=>
  ["src/nginx_module/LocationConfig/AutoGeneratedStruct.h",
   "src/nginx_module/MainConfig/AutoGeneratedStruct.h"],
 "src/nginx_module/ContentHandler.c"=>
  ["src/cxx_supportlib/AppTypeDetector/CBindings.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "src/nginx_module/Configuration.h",
   "src/nginx_module/ContentHandler.h",
   "src/nginx_module/LocationConfig/AutoGeneratedStruct.h",
   "src/nginx_module/MainConfig/AutoGeneratedStruct.h",
   "src/nginx_module/StaticContentHandler.h"],
 "src/nginx_module/ContentHandler.h"=>
  ["src/cxx_supportlib/AppTypeDetector/CBindings.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/nginx_module/LocationConfig/AutoGeneratedCreateFunction.c"=>
  [],
 "src/nginx_module/LocationConfig/AutoGeneratedHeaderSerialization.c"=>
  [],
 "src/nginx_module/LocationConfig/AutoGeneratedManifestGeneration.c"=>
  ["src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.h"],
 "src/nginx_module/LocationConfig/AutoGeneratedMergeFunction.c"=>
  [],
 "src/nginx_module/LocationConfig/AutoGeneratedStruct.h"=>
  [],
 "src/nginx_module/MainConfig/AutoGeneratedCreateFunction.c"=>
  [],
 "src/nginx_module/MainConfig/AutoGeneratedManifestGeneration.c"=>
  ["src/cxx_supportlib/FileTools/PathManipCBindings.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/nginx_module/ConfigGeneral/ManifestGeneration.h"],
 "src/nginx_module/MainConfig/AutoGeneratedStruct.h"=>
  [],
 "src/nginx_module/StaticContentHandler.c"=>
  ["src/nginx_module/StaticContentHandler.h"],
 "src/nginx_module/StaticContentHandler.h"=>
  [],
 "src/nginx_module/ngx_http_passenger_module.c"=>
  ["src/cxx_supportlib/Constants.h",
   "src/nginx_module/Configuration.h",
   "src/nginx_module/LocationConfig/AutoGeneratedStruct.h",
   "src/nginx_module/MainConfig/AutoGeneratedStruct.h"],
 "src/nginx_module/ngx_http_passenger_module.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppTypeDetector/CBindings.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/JsonTools/CBindings.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.h",
   "src/cxx_supportlib/Utils/ClassUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/WatchdogLauncher.h",
   "src/cxx_supportlib/WrapperRegistry/CBindings.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "src/ruby_native_extension/passenger_native_support.c"=>
  [],
 "src/schema_printer/SchemaPrinterMain.cpp"=>
  ["src/agent/Core/ApiServer.h",
   "src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Config.h",
   "src/agent/Core/ConfigChange.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SecurityUpdateChecker.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/Shared/ApiAccountUtils.h",
   "src/agent/Shared/ApiServerUtils.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/agent/Watchdog/ApiServer.h",
   "src/agent/Watchdog/Config.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/llversion.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp"],
 "test/cxx/Base64DecodingTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/CachedFileStatTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/ErrorTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/NestedSchemaTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/SchemaTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/StoreTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/SubSchemaTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ConfigKit/TranslationTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/PrefixTranslator.h",
   "src/cxx_supportlib/ConfigKit/TableTranslator.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/ApplicationPool/OptionsTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/ApplicationPool/PoolTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/ApplicationPool/ProcessTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/ControllerTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/ApplicationPool/TestSession.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/ResponseCacheTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SecurityUpdateCheckerTest.cpp"=>
  ["src/agent/Core/SecurityUpdateChecker.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/Crypto.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/ConfigTest.cpp"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/DirectSpawnerTest.cpp"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/Core/SpawningKit/SpawnerTestCases.cpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/HandshakePerformTest.cpp"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/HandshakePrepareTest.cpp"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/JourneyTest.cpp"=>
  ["src/agent/Core/SpawningKit/Journey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/SmartSpawnerTest.cpp"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Config.h",
   "src/cxx_supportlib/LoggingKit/Context.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/Core/SpawningKit/SpawnerTestCases.cpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/SpawningKit/SpawnerTestCases.cpp"=>
  [],
 "test/cxx/Core/SpawningKit/UserSwitchingRulesTest.cpp"=>
  ["src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/Core/TelemetryCollectorTest.cpp"=>
  ["src/agent/Core/ApplicationPool/AbstractSession.h",
   "src/agent/Core/ApplicationPool/BasicGroupInfo.h",
   "src/agent/Core/ApplicationPool/BasicProcessInfo.h",
   "src/agent/Core/ApplicationPool/Common.h",
   "src/agent/Core/ApplicationPool/Context.h",
   "src/agent/Core/ApplicationPool/Group.h",
   "src/agent/Core/ApplicationPool/Options.h",
   "src/agent/Core/ApplicationPool/Pool.h",
   "src/agent/Core/ApplicationPool/Process.h",
   "src/agent/Core/ApplicationPool/Session.h",
   "src/agent/Core/ApplicationPool/Socket.h",
   "src/agent/Core/Controller.h",
   "src/agent/Core/Controller/AppResponse.h",
   "src/agent/Core/Controller/Client.h",
   "src/agent/Core/Controller/Config.h",
   "src/agent/Core/Controller/Request.h",
   "src/agent/Core/Controller/TurboCaching.h",
   "src/agent/Core/ResponseCache.h",
   "src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/DirectSpawner.h",
   "src/agent/Core/SpawningKit/DummySpawner.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Factory.h",
   "src/agent/Core/SpawningKit/Handshake/BackgroundIOCapturer.h",
   "src/agent/Core/SpawningKit/Handshake/Perform.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/PipeWatcher.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/SmartSpawner.h",
   "src/agent/Core/SpawningKit/Spawner.h",
   "src/agent/Core/SpawningKit/UserSwitchingRules.h",
   "src/agent/Core/TelemetryCollector.h",
   "src/agent/Shared/ApplicationPoolApiKey.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/AppLocalConfigFileUtils.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/SchemaUtils.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/Hooks.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/LveLoggingDecorator.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/ServerKit/url_parser.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/AsyncSignalSafeUtils.h",
   "src/cxx_supportlib/Utils/Curl.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Lock.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/CxxTestMain.cpp"=>
  ["src/agent/Shared/Fundamentals/AbortHandler.h",
   "src/agent/Shared/Fundamentals/Initialization.h",
   "src/agent/Shared/Fundamentals/Utils.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h",
   "test/tut/tut_reporter.h"],
 "test/cxx/DataStructures/LStringTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/DataStructures/StringKeyTableTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/DataStructures/StringMapTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/DateParsingTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/DateParsing.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/FileChangeCheckerTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/CachedFileStat.hpp",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/FileChangeChecker.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/FileDescriptorTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/FileTools/PathSecurityCheckTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathSecurityCheck.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/IOTools/BufferedIOTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/IOTools/IOUtilsTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/IOTools/MessageIOTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/IOTools/MessageSerializationTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageSerialization.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/MemoryKit/MbufTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/MemoryKit/PallocTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/MessagePassingTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/MessagePassing.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/ChannelTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/CookieUtilsTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/ServerKit/CookieUtils.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/FileBufferedChannelTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/HeaderTableTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/HttpServerTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/LString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/BufferedIO.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/Integrations/LibevJsonUtils.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/MemoryKit/palloc.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/HeaderTable.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParser.h",
   "src/cxx_supportlib/ServerKit/HttpChunkedBodyParserState.h",
   "src/cxx_supportlib/ServerKit/HttpClient.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParser.h",
   "src/cxx_supportlib/ServerKit/HttpHeaderParserState.h",
   "src/cxx_supportlib/ServerKit/HttpRequest.h",
   "src/cxx_supportlib/ServerKit/HttpRequestRef.h",
   "src/cxx_supportlib/ServerKit/HttpServer.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/HttpConstants.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/ServerKit/ServerTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/Algorithms/MovingAverage.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/MemoryKit/mbuf.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SafeLibev.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/ServerKit/Channel.h",
   "src/cxx_supportlib/ServerKit/Client.h",
   "src/cxx_supportlib/ServerKit/ClientRef.h",
   "src/cxx_supportlib/ServerKit/Config.h",
   "src/cxx_supportlib/ServerKit/Context.h",
   "src/cxx_supportlib/ServerKit/Errors.h",
   "src/cxx_supportlib/ServerKit/FdSourceChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedChannel.h",
   "src/cxx_supportlib/ServerKit/FileBufferedFdSinkChannel.h",
   "src/cxx_supportlib/ServerKit/Hooks.h",
   "src/cxx_supportlib/ServerKit/Server.h",
   "src/cxx_supportlib/ServerKit/llerrors.h",
   "src/cxx_supportlib/ServerKit/llhttp.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/SpawnEnvSetupperTest.cpp"=>
  ["src/agent/Core/SpawningKit/Config.h",
   "src/agent/Core/SpawningKit/Config/AutoGeneratedCode.h",
   "src/agent/Core/SpawningKit/Context.h",
   "src/agent/Core/SpawningKit/Exceptions.h",
   "src/agent/Core/SpawningKit/Handshake/Prepare.h",
   "src/agent/Core/SpawningKit/Handshake/Session.h",
   "src/agent/Core/SpawningKit/Handshake/WorkDir.h",
   "src/agent/Core/SpawningKit/Journey.h",
   "src/agent/Core/SpawningKit/Result.h",
   "src/agent/Core/SpawningKit/Result/AutoGeneratedCode.h",
   "src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/ConfigKit/Common.h",
   "src/cxx_supportlib/ConfigKit/ConfigKit.h",
   "src/cxx_supportlib/ConfigKit/DummyTranslator.h",
   "src/cxx_supportlib/ConfigKit/Schema.h",
   "src/cxx_supportlib/ConfigKit/Store.h",
   "src/cxx_supportlib/ConfigKit/Translator.h",
   "src/cxx_supportlib/ConfigKit/Utils.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringKeyTable.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/JsonTools/JsonUtils.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/SystemMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/AnsiColorConstants.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/SpeedMeter.h",
   "src/cxx_supportlib/Utils/Timer.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/WrapperRegistry/Entry.h",
   "src/cxx_supportlib/WrapperRegistry/Registry.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/StaticStringTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/StrIntTools/StrIntUtilsTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/StrIntTools/TemplateTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/DataStructures/StringMap.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/Template.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/SystemTools/ProcessMetricsCollectorTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/StrIntTools/StringScanning.h",
   "src/cxx_supportlib/SystemTools/ProcessMetricsCollector.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/SystemTools/SystemTimeTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/TestSupport.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/support/valgrind.h",
   "test/tut/tut.h"],
 "test/cxx/TestSupport.h"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/tut/tut.h"],
 "test/cxx/UtilsTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/FileTools/PathManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/cxx/VariantMapTest.cpp"=>
  ["src/cxx_supportlib/Algorithms/Hasher.h",
   "src/cxx_supportlib/BackgroundEventLoop.h",
   "src/cxx_supportlib/Constants.h",
   "src/cxx_supportlib/DataStructures/HashedStaticString.h",
   "src/cxx_supportlib/Exceptions.h",
   "src/cxx_supportlib/FileDescriptor.h",
   "src/cxx_supportlib/FileTools/FileManip.h",
   "src/cxx_supportlib/IOTools/IOUtils.h",
   "src/cxx_supportlib/IOTools/MessageIO.h",
   "src/cxx_supportlib/InstanceDirectory.h",
   "src/cxx_supportlib/LoggingKit/Assert.h",
   "src/cxx_supportlib/LoggingKit/Forward.h",
   "src/cxx_supportlib/LoggingKit/Logging.h",
   "src/cxx_supportlib/LoggingKit/LoggingKit.h",
   "src/cxx_supportlib/ProcessManagement/Spawn.h",
   "src/cxx_supportlib/ProcessManagement/Utils.h",
   "src/cxx_supportlib/RandomGenerator.h",
   "src/cxx_supportlib/ResourceLocator.h",
   "src/cxx_supportlib/SecurityKit/MemZeroGuard.h",
   "src/cxx_supportlib/StaticString.h",
   "src/cxx_supportlib/StrIntTools/StrIntUtils.h",
   "src/cxx_supportlib/SystemTools/SystemTime.h",
   "src/cxx_supportlib/SystemTools/UserDatabase.h",
   "src/cxx_supportlib/Utils.h",
   "src/cxx_supportlib/Utils/FastStringStream.h",
   "src/cxx_supportlib/Utils/IniFile.h",
   "src/cxx_supportlib/Utils/ScopeGuard.h",
   "src/cxx_supportlib/Utils/VariantMap.h",
   "src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/cxx/TestSupport.h",
   "test/tut/tut.h"],
 "test/oxt/backtrace_test.cpp"=>
  ["src/cxx_supportlib/oxt/backtrace.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/backtrace_enabled.hpp",
   "src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_disabled.hpp",
   "src/cxx_supportlib/oxt/detail/tracable_exception_enabled.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "src/cxx_supportlib/oxt/tracable_exception.hpp",
   "test/oxt/counter.hpp",
   "test/tut/tut.h"],
 "test/oxt/counter.hpp"=>
  [],
 "test/oxt/dynamic_thread_group_test.cpp"=>
  ["src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/dynamic_thread_group.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "test/oxt/counter.hpp",
   "test/tut/tut.h"],
 "test/oxt/oxt_test_main.cpp"=>
  ["src/cxx_supportlib/oxt/initialize.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "test/tut/tut.h",
   "test/tut/tut_reporter.h"],
 "test/oxt/spin_lock_test.cpp"=>
  ["src/cxx_supportlib/oxt/spin_lock.hpp",
   "test/tut/tut.h"],
 "test/oxt/syscall_interruption_test.cpp"=>
  ["src/cxx_supportlib/oxt/detail/context.hpp",
   "src/cxx_supportlib/oxt/macros.hpp",
   "src/cxx_supportlib/oxt/spin_lock.hpp",
   "src/cxx_supportlib/oxt/system_calls.hpp",
   "src/cxx_supportlib/oxt/thread.hpp",
   "test/tut/tut.h"]}
